*******************************************************************************
*REVISION HISTORY:
*
*DATE			REV. NO.	DESCRIPTION
*
*June 26, 2001	V2.00		- Revamp Flash Loader.
*					- Added G command
*					- Support for RAM as Program Memroy
*					- Support for S2 records
*					- Enhanced Program/Verify command
*
*July 13, 2001	V2.01		- Fix bug
*					- Program S2 in non-banked mode from $8000 to $FFFF
*					- Dump 16k Program Memory from requested page
*					- Added check to bypass programming Ram location $800 to $C00
*
*December 2, 2001	V2.02		- Added send "*" at end of S8 or S9 termination and programming
*
*Author: Exequiel Rarama
********************************************************************************
;Compile using MiniIDE
;
#INCLUDE REGS.INC             		;register block addresses

EEPROM  equ     $f000           	;68HC812A4 internal EEPROM in SCM
RAM     equ     $0800           	;68HC812A4 internal RAM
STACK   equ     $0c00           	;Stack at top of internal ram

;SCI Variables
scimask         equ     %00101100       ;RIE - SCI Interrupt enable
scimask1        equ     $0c             ;TE - Transmit Enable, RE - Receiver Enable
RDRFflag        equ     %00100000       ;RDRF - Receive Data Register Full flag
TDREflag        equ     %10000000       ;TDRE - Transmit Data Register Empty flag

;Baud rate definitions
;MCLK=8Mzh
BAUD9600        equ     52      	;(baud) 9600 baud with 16 Mhz crystal
BAUD14400       equ     35      	;(baud) 14400 baud with 16 Mhz crystal
BAUD19200       equ     26      	;(baud) 19200 baud with 16 Mhz crystal
BAUD38400       equ     13      	;(baud) 19200 baud with 16 Mhz crystal
;
true		equ	$ff
;
;------------------------------------------------------------------------------
LED_ON	equ	$40
LED_OFF	equ	$40

S0REcType       equ     '0'     	;ASCII '0' used as S0 record type indicator
S1REcType       equ     '1'     	;ASCII '1' used as S1 record type indicator
S2REcType	equ	'2'		;ASCII '2' used as S2 record type indicator
S8REcType       equ     '8'     	;ASCII '8' used as S8 record terminator
S9REcType       equ     '9'     	;ASCII '9' used as S9 record type indicator

EEON		equ	%01

SRAM		equ	$7000		;start of external SRAM, when enabled
DWIN		equ	$7000		;start of data window
ENDDWIN		equ	$7fff		;end of data window
FLASH		equ	$8000		;start of Flash
PWIN		equ	$8000		;start of program window
ENDPWIN		equ	$bfff		;end of program window
LDP128		equ	$1f		;last 4K page of 128K data space
LDP512		equ	$7f		;last 4K page of 512K data space
LPP128		equ	$07		;last 16K page of 128K program space
LPP512		equ	$1f		;last 16K page of 512K program space
;
PRGmemory	equ	$8000		;beginning of Program Flash
PRGEnd		equ	$C000		;end of Program Flash
;-----------------------------------------------------------------------------
;
        org     RAM

;Ram Variables
page_address	ds.b	1
abs_address	ds.b	2
first_err	ds.b	1
good_data	ds.b	1
bad_data	ds.b	1
counter		ds.b	1

S_ADR		ds.b	4

BYTE		ds.b	1		;used for data polling

temp		ds.b	1
temp1		ds.b	1

CURPAGE		ds.b	1		;current page counter
REFPAGE		ds.b	1		;referance of last page before call on $C000 - $FFFF Page
MEM_SIZE	ds.b	2		;memory size (128 or 512)
LAST_PPAGE	ds.b	1		;last page of Flash
LAST_DPAGE	ds.b	1		;last page of RAM

ERROR		ds.b	1		;Flash operation status
WRITE_MODE	ds.b	1		;write or verify
BANKING		ds.b	1		;banking or non-banking mode
flashtype       ds.b    1      		;amd or st flash
mem_type	ds.b	1		;size for unknown device

RAMCode		equ	mem_type+1

;-----------------------------------------------------------------------------
        org     EEPROM
;
Start                       
        sei                     	;Disable Any interrupts

;Initialize Stack
        lds     #STACK          	;initialize stack pointer

;Initialize Serial Communication Interface
        movb    #scimask1,SC0CR2        ;enable SCI 0 rcvr. & xmtr.
        movb    #0,SC0CR1
;
        movw    #BAUD9600,SC0BDH        ;Set baud rate to 9600
        ldab    SC0SR1                  ;read register to clear flag RDRF
        ldab    SC0DRL                  ;read receive buffer

	movb	#$40,DDRT		;make port t bit 6 o/p

;-------------------------------------------------------------------------------
FlashLoad
        clr     COPCTL    		;Disable watchdog timer              

;Set up external memory control registers
	movb	#$04,PEAR		;enable read/write signal
	movb	#$B0,MODE		;normal exp. narrow
	movb	#$C0,WINDEF		;enable data & program windows
   	movb  	#$07,MXAR 	      	;enable A16, A17, A18
	movb	#$30,CSCTL0		;enable CSD & CSP0
	movb	#$10,CSCTL1		;CSD covers lower half of memory map

	ldx	#512			;default memory size is 512K
	stx	MEM_SIZE
	movb	#LPP512,LAST_PPAGE
	movb	#LDP512,LAST_DPAGE
	movb	#true,BANKING		;default to banked mode

        ldx	#CrlfStr	  	;go to next line
        jsr     OutStr		   	;display it.

        jsr     read_device 		;sets flashtype variables
	bsr	ShowMenu		;display menu in terminal window

main
	bsr     ProcessCommand		;this is the program loop
	bsr	ShowPrompt
	bra	main

;----------------------------------------------------------------------------------------
ShowMenu
	ldx	#M_msg
	jsr	OutStr

ShowPrompt
	ldx	#M_PROMPT
	jsr	OutStr
	rts

ProcessCommand
        jsr     getchar		  	;get the command from the user
        jsr     putchar		  	; and echo it.
	psha

        ldx	#CrlfStr	  	;go to next line
        jsr     OutStr		   	;display it.
	
	pula
	oraa	#$20

	clr	PORTT			;LED off

	cmpa	#'m'			;show menu
	beq	ShowMenu

	cmpa	#'j'
	lbeq	choose_unk_dev_size	;Toggle device size	
  
        cmpa    #'e'			;erase Flash
	lbeq	EraseFlash

        cmpa    #'p'			;program s-record into Flash
	lbeq	ProgramFlash

        cmpa    #'v'			;verify s-record
	lbeq	VerSRecords

        cmpa    #'a'            	;set baud = 9600
	lbeq	BaudChange

        cmpa    #'b'            	;set baud = 14k4
	lbeq	BaudChange
    
	cmpa    #'c'            	;set baud = 19k2
	lbeq	BaudChange

	cmpa    #'d'            	;set baud = 38k4
	lbeq	BaudChange

	cmpa    #'n'			;set banking/non-banking mode
	lbeq	BankType

	cmpa	#'g'
	lbeq	CopyToRAM		;Copy small program to RAM and execute program there
	
	cmpa	#'f'
	lbeq	dump_page

	rts

;==============================================================================
;----------------------------
;Toggle Bank or Non Bank Mode
;----------------------------
BankType
	com	BANKING			;toggle bank/non-banked mode
	bra	m_bank

;-----------------------------------------
;Memory Type and Banked or Non Banked Mode
;-----------------------------------------
choose_unk_dev_size			;Unknown device type and is assume to be RAM
	ldaa	flashtype
	bne	MemoryType

	ldaa	mem_type
	beq	set_512K_size

        ldx     #128            	;memory size is 128K
	stx	MEM_SIZE
        movb    #LPP128,LAST_PPAGE
	clr	mem_type

	ldx	#M_UN
	jsr	OutStr
	
	bra	MEM128


set_512K_size
	movb	#%1,mem_type		;512K RAM Memory

        ldy     #512                    ;set size
	sty	MEM_SIZE		;memory size is 512K
        movb    #LPP512,LAST_PPAGE

	ldx	#M_UN
	bra	show_type

MemoryType
        brset   flashtype,%0001,amd_type	;erase AMD 512K memory
        brset   flashtype,%0010,amd_type	;erase AMD 128K memory
        brset   flashtype,%0100,st_type		;erase ST 512K memory
        brset   flashtype,%1000,st_type		;erase ST 128K memory

amd_type
	ldx	#M_AMD
	bra	show_type

st_type
	ldx	#M_ST

show_type
	jsr	OutStr

        ldx     MEM_SIZE
        cpx     #512
        bne     MEM128

	ldx	#M_MEM512		;512K memory type
	bra	MEM512

MEM128
        ldx	#M_MEM128		;128K memory type

MEM512
	jsr     OutStr			;Send messages

	ldaa	flashtype
	beq	unknown_mem	
	ldx	#M_FLASH
	bra	m_flash10

unknown_mem
	ldx	#M_UNK_MEM

m_flash10
	jsr     OutStr			;Send messages

m_bank
	tst	BANKING
	bne	BankingOn

	ldx	#M_BANKING_OFF
	bra	BankingOFF

BankingON
	ldx	#M_BANKING_ON

BankingOFF
	jsr	OutStr
	rts

;==============================================================================
;-----------------
;Baud Rate Routine
;-----------------
BaudChange
	psha

        ldx	#M_BAUD1
	jsr	OutStr

	ldy	#$ffff

delay
	dey
	bne	delay	

	pula	
	cmpa	#'a'
	beq	BD9600

	cmpa	#'b'
	beq	BD14400

	cmpa	#'c'
	beq	BD19200

	cmpa	#'d'
	beq	BD38400

	rts

BD9600
        movw    #BAUD9600,SC0BDH 	;Set baud rate to 9600
        rts

BD14400
        movw    #BAUD14400,SC0BDH	;Set baud rate to 14400
	rts

BD19200
        movw    #BAUD19200,SC0BDH       ;Set baud rate to 19200
	rts

BD38400
        movw    #BAUD38400,SC0BDH       ;Set baud rate to 38400
	rts

;==============================================================================
;----------------------------------
;Verify Flash memory with s-records
;----------------------------------
VerSRecords
	clr	WRITE_MODE		;select verify mode

	ldx	#M_SEND_S19
	jsr	OutStr

	clr	ERROR			;no errors yet
	clr	first_err

        jsr	Burn_s_records

	tst	ERROR
	bne	PCV_Fail

	ldx	#M_VERIFY_OK
	bra	PCV_SS

PCV_Fail
	ldx	#M_VERIFY_FAIL
	jsr	OutStr

	ldx	#M_ERR_PAGE
	jsr	OutStr

	bra	PV_Fail
	

PCV_SS
	jsr	OutStr
	rts

;==============================================================================
;---------------------
;Program Flash Routine
;---------------------
ProgramFlash
	ldx	#M_PROMPT1
	jsr	OutStr

	clr	ERROR			;no errors yet
	clr	first_err

	movb	#true,WRITE_MODE	;select non-verify mode
        bsr	Burn_s_records

	tst	ERROR
	bne	PCP_Fail

	ldx	#M_PROG_OK
	bra	PCP_SS

PCP_Fail
	ldx	#M_PROG_FAIL
	jsr	OutStr

PV_Fail
	ldaa	page_address
	jsr	Hex2Dec
	bsr	Dump_data

	ldx	#M_ERR_ADDRESS
	jsr	OutStr

	ldd	abs_address	
	bsr	Send_word

	ldx	#M_DATA
	jsr	OutStr

	ldaa	bad_data
	jsr	Hex2Dec
	bsr	Dump_data

	ldx	#M_DATA1
	jsr	OutStr

	ldaa	good_data
	jsr	Hex2Dec
	bsr	Dump_data

	rts

PCP_SS:
	jsr	OutStr
	rts


Send_word
	psha
	xgdx
	pula

	jsr	Hex2Dec
	bsr	Dump_data

	xgdx
	tba
	jsr	Hex2Dec
	bsr	Dump_data

	rts

Dump_data
	ldaa	temp
	jsr	putchar
	ldaa	temp1
	jsr	putchar
	rts


Dump_word
	bra	Send10

SendAddress			;Reg X contains the address
	pshx
	puld			;copy address to Reg D

Send10
	jsr	Hex2Dec
	bsr	Dump_data

	tba			;Exchange Data
	jsr	Hex2Dec
	bsr	Dump_data

	rts

#INCLUDE EEFLASH.ASM

;==============================================================================
;-----------------------------------------------
;Read Flash Device Type and Manufacturer Routine
;-----------------------------------------------
read_device
        clr     flashtype
	movb	#$1,PPAGE
	movb	#$aa,$9555
	clr	PPAGE

	movb	#$55,$aaaa
	movb	#$1,PPAGE
        movb    #$90,$9555
	clr	PPAGE

        ldx     #$8000
        ldaa    0,x

        cmpa    #$01                    ;check if AMD made memory
        beq     amd_made

        movb    #$aa,$8555
        movb    #$55,$82aa
        movb    #$90,$8555

        ldx     #$8000
        ldaa    0,x

        cmpa    #$20                    ;check if st made memory
        beq     st_made

        bra     unknown_dev

amd_made
        ldx     #$8001
        ldaa    0,x

        cmpa    #$A4                    ;check if AMD 512K flash memory
        beq     amd_dev512

        cmpa    #$20                    ;check if AMD 128K flash memory
        beq     amd_dev128

        bra     unknown_dev

st_made
        ldx     #$8001
        ldaa    0,x

        cmpa    #$E2                    ;check if ST 512K flash memory
        beq     st_512K

        cmpa    #$20                    ;check if ST 128K flash memory
        beq     st_128K

        bra     unknown_dev

amd_dev512
        ldy     #512                    ;set size
	sty	MEM_SIZE
        movb    #LPP512,LAST_PPAGE

        movb    #%0001,flashtype
        movb    #$f0,$8200              ;reset AMD 040 flash
        bra     amd_dev

amd_dev128
        ldy     #128                    ;set size
	sty	MEM_SIZE
        movb    #LPP128,LAST_PPAGE

        movb    #%0010,flashtype
        movb    #$f0,$8200              ;reset AMD 010 flash
        bra     amd_dev

st_512K
	ldx	#512			;memory size is 512K
	stx	MEM_SIZE
	movb	#LPP512,LAST_PPAGE
        movb    #%0100,flashtype
        movb    #$f0,$8200              ;reset ST 040 flash

        bra     st_dev

st_128K
        ldx     #128            	;memory size is 512K
	stx	MEM_SIZE
        movb    #LPP128,LAST_PPAGE
        movb    #%1000,flashtype
        movb    #$f0,$8200              ;reset ST 010 flash

        bra     st_dev

unknown_dev
        clr     flashtype
        ldy     #128                    ;set size to default
	sty	MEM_SIZE
	clr	mem_type		;128K RAM Memory

        ldx	#M_UN
	jsr	OutStr

	ldx	#M_UN2
	bra	read_ex

amd_dev
	ldx	#M_AMD
	bra	read_ex

st_dev	
	ldx	#M_ST

read_ex
	jsr	OutStr

        rts

;==============================================================================
;-----------------------------------------
;Copy small program to Ram and execute it.  
;-----------------------------------------
CopyToRAM
	movb	LAST_PPAGE,PPAGE	;make sure PPAGE is set for Last Page

	ldx	#M_COPY_RAM
	jsr	OutStr

        ldx     #ProgCode               ;Point to the start of code
        ldy     #RAMcode                ;memory ram location to save boot.
        ldd     #code_len               ;End of boot code

MoveMore
        movb    1,x+,1,y+               ;move byte to ram
        dbne    d,MoveMore              ; until done

	jmp	RAMcode

ProgCode
	bclr	INITEE,EEON		;Disable eeprom
	bclr	INITEE,EEON

        lds     #STACK          	;initialize stack pointer	

	ldx	$FFFE			;Go execute Program
	jmp	0,x

code_len 	equ    ($-ProgCode)

;-----------------------------------------------------------------------------
dump_page
	ldx	#M_PAGE
        jsr     OutStr

	jsr	GetPage
	staa	PPAGE

DumpBlock
	ldx	#PRGmemory	;point to beginning of ram

Dump10
	jsr	SendAddress
	clr	counter		;initialize counter

Dump20
	ldaa	#$20		;place space
	jsr	putchar

	ldd	2,x+		;increment address
	jsr	Dump_word	;Send address

	cpx	#PRGend
	beq	DumpEnd

	inc	counter			;increment counter
	inc	counter

	ldab	counter
	cmpb	#16
	bne	Dump20

	pshx			;Save address
	ldx	#CrlfStr	;Send carriage and line feed character
        jsr     OutStr		;
	pulx			;Restore address
	
	bra	Dump10

DumpEnd
	rts

;==============================================================================
;---------------------------------------------------
;Extracts load address and returns it in register Y.
;---------------------------------------------------
GetAddress
	pshb				;preserve byte count
	bsr	GetHexByte	
	staa	temp1
	bsr	GetHexByte
	tab
	ldaa	temp1
	xgdy
	pulb
	rts	

;-----------------------------------------------------------------------------
GetHexByte				;Value in Reg A
        bsr     getchar   	   	;get the upper nibble from the SCI

        bsr     CvtHex       	   	;convert the ascii hex to bin
        asla
        asla
        asla
        asla
        anda    #$f0
        staa    temp

        bsr     getchar         	;get lower nibble from the SCI
        bsr     CvtHex         		;convert to bin
        adda    temp            	;add it to upper nibble

        rts

CvtHex
        suba    #'0'            	;subtract ascii '0' from hex char
        cmpa    #$9             	;was it a decimal digit?
        bls     CvtHexRtn       	;yes, ok as is
        suba    #$7             	;no, it was an ascii hex letter ('A'-'F')

CvtHexRtn
        rts

;------------------------------------------------------------------------------
GetPage					;Value in Reg A
        bsr     getchar   	   	;get the upper nibble from the SCI
	bsr	IsHex
        bne     not_hex			;no

        bsr     CvtHex       	   	;convert the ascii hex to bin
        asla
        asla
        asla
        asla
        anda    #$f0
        staa    temp

        bsr     getchar         	;get lower nibble from the SCI
	bsr	IsHex
        bne     not_hex			;no

        bsr     CvtHex         		;convert to bin
        adda    temp            	;add it to upper nibble

        rts

not_hex
	ldx	#M_HEX
	jsr	OutStr

	ldx	#M_PAGE
        jsr     OutStr

	bra	GetPage

IsHex
        cmpa	#'0'            ;less than ascii hex zero?
        blo     NotHex          ;yes, char is not hex, return a non zero ccr indication
        cmpa    #'9'            ;less than or equal to ascii 9?
        bls     IsHex1          ;yes, char is hex, return a zero ccr indication
        cmpa    #'A'            ;less than an ascii hex 'A'?
        blo     NotHex          ;yes, char is not hex
        cmpa    #'F'            ;less than or equal to ascii hex 'F'?
        bhi     NotHex          ;yes, char is hex

IsHex1
        orcc    #%0100          ;no, return a zero ccr conditon

NotHex
        rts

;------------------------------------------------------------------------------
Hex2Dec
	psha
	asra
	asra
	asra
	asra
	bsr	CvtBin2Dec
	staa	temp

	pula
	bsr	CvtBin2Dec
	staa	temp1
	rts

CvtBin2Dec
	anda	#$0f
	cmpa	#$9
	bls	CvtDec
	adda	#$7

CvtDec
	adda	#$30
	rts


;-----------------------------------------------------------------------------
getchar
	brclr   SC0SR1,RDRFflag,*	;loop waiting for the RDRF bit to be set
        ldaa    SC0SR1          	;read register to clear flag RDRF
        ldaa    SC0DRL          	;read receive buffer
        rts

OutStr
        ldaa    1,x+            	;get a char, advance pointer, null?
        beq     OutStrDone      	;yes, return
        bsr     putchar         	;no, sent it out
        bra     OutStr          	;go send next char

OutStrDone
        rts

putchar
        brclr   SC0SR1,TDREflag,*  	;loop waiting for the TDRE bit to be set
        staa    SC0DRL             	;send char
        rts

;------------------------------------------------------------------------------
CrlfStr         dc.b    $0a,$0d,$0

M_msg	        dc.b     $0a,$0d,$0a,$0d
M_UNDLN1	dc.b	'------------------------------------------------',$0a,$0d
M_HEADER1       dc.b    ' 812 NARROW MODE AMD/ST FLASH UTILITY  VER. 2.02 ',$0a,$0d
M_UNDLN2        dc.b    '------------------------------------------------',$0a,$0d,$0a,$0d

M_DUMP		dc.b	'F => Dump 16K Block Program Memory from Requested Page',$0a,$0d
M_EXECUTE	dc.b	'G => Execute Program Pointed to by Reset Vector',$0a,$0d
M_DEVICE	dc.b	'J => Toggle Unknown Proram Memory size - 128K/512K',$0a,$0d
M_BANKING	dc.b    'N => Toggle Banked/Non-banked Program Memory Mode',$0a,$0d
M_MENU		dc.b	'M => Show Menu',$0a,$0d
M_ERASE         dc.b    'E => Erase entire Flash Memory',$0a,$0d
M_PROGRAM       dc.b    'P => Program S1 or S2 record into Program Memory',$0a,$0d
M_VERIFY        dc.b    'V => Verify S-records in Program memory',$0a,$0d
M_BAUD          dc.b    'Baud choice A->9600, B->14K4, C->19K2, D->38K4',$0a,$0d,0
M_PROMPT        dc.b    $0a,$0d,'? ',0

M_PROMPT1	dc.b    'Send S-record file now ',$0a,$0d,0
M_SEND_S19	dc.b	'Send s-record file to be verified',$0a,$0d,0
M_VERIFY_FAIL   dc.b    's-record verification failed at ',0
M_VERIFY_OK     dc.b    's-record verified OK',0

M_ERASE_OK	dc.b    'Flash erased OK',0
M_ERASE_FAIL    dc.b    'Could not erase Flash',0
M_ERASING	dc.b    'erasing Flash now...',0
M_PROG_OK       dc.b    'Flash programmed OK',0
M_PROG_FAIL     dc.b    'Flash programming failed @ '
M_ERR_PAGE	dc.b	'PAGE $',0
M_ERR_ADDRESS	dc.b	' ADDRESS $',0
M_DATA		dc.b	$0a,$0d,'Data is $',0
M_DATA1		dc.b	' Should be $',0

M_COPY_RAM	dc.b	$0a,$0d,'Run',$0a,$0d,0

M_MEM128	dc.b    ' 128K ',0
M_MEM512	dc.b    ' 512K ',0
M_BANKING_OFF	dc.b    'Non-banked Mode',0
M_BANKING_ON	dc.b    'Banked Mode',0
M_FLASH		dc.b	'- Flash ',0
M_UNK_MEM	dc.b	'- RAM? ',0

M_ST            dc.b    'ST Flash Device',0
M_AMD           dc.b    'AMD Flash Device',0
M_UN            dc.b    'Unknown Device',0
M_UN2           dc.b    ' - Defaulting to 128K Memory size',0

M_BAUD1         dc.b    'Change Baud now',$0a,$0d,0

M_FlashType	dc.b	'F => Memory Size and Mode',$0a,$0d,0
M_PAGE		dc.b	$0a,$0d,'Page XX?',$0a,$0d,0
M_HEX		dc.b	$0a,$0d,'Not a Hex character',$0a,$0d,0

;------------------------------------------------------------------------------
	org	$FFFE
	
	dc.w	Start


