*******************************************************************************
*REVISION HISTORY:
*
*DATE			REV. NO.	DESCRIPTION
*
*June 26, 2001	V2.00		- Revamp Flash Loader.
*					- Added G command
*					- Support for RAM as Program Memroy
*					- Support for S2 records
*					- Enhanced Program/Verify command
*
*July 13, 2001	V2.01		- Fix bug
*					- Program S2 in non-banked mode from $8000 to $FFFF
*					- Dump 16k Program Memory from requested page
*					- Added check to bypass programming Ram location $800 to $C00
*
*December 2, 2001	V2.02		- Added send "*" at end of S8 or S9 termination and programming
*
*Author: Exequiel Rarama
********************************************************************************
;Compile using MiniIDE
;
;==============================================================================
;-------------------------------
;Read S-records and Burn Routine
;-------------------------------
Burn_s_records
	clr	ERROR				;no errors yet

Get_S
	jsr	GetChar			;look for S record

	cmpa	#'S'
	bne	Get_S

	jsr	GetChar
	cmpa	#S1REcType		;if S1 record then
	lbeq	Load1

	cmpa	#S2REcType		;if S2 record then
	beq	Load2

	cmpa	#S8REcType
	beq	S2_end

	cmpa	#S9REcType		;if S9 record then
	bne	Get_S

	jsr	GetHexByte		;get byte count
	tab
	subb	#2
	jsr	GetAddress		;get execution address

Load9
	jsr	GetHexByte		;get checksum byte (and ignore it)
	decb
	bne	Load9

        ldaa	#'*'		;the ascii asterisk is the pace char
        jsr	putchar		;tell the host it is ok to send the nex S-record

	lbra	PSX

S2_end
	jsr	GetHexByte		;get byte count
	tab
	
check_s2_end
	jsr	GetHexByte
	decb
	bne	check_s2_end

        ldaa	#'*'		;the ascii asterisk is the pace char
        jsr	putchar		;tell the host it is ok to send the nex S-record

	lbra	PSX

Load2
	jsr	GetHexByte		;S2 record, so get byte count
	tab
	subb	#4			;adjust count for (24bit)address and checksum bytes
	pshb				;save record lenght

	jsr	GetHexByte		;get upper part of address (A21-A16)
	psha				; save top of address
	jsr	GetAddress		;get lower part of address (A15-A0)

	sty	S_ADR

	xgdy

	psha				; save upper part of address range (A15-A12)
	anda	#$3F			; mask off A15 and A14 this allows to place the address 
					; into range from $8000 to $bfff
	addd	#$8000			; add Flash ROM base address offset $8000+(A13-A00)

	xgdy				;Reg Y contains the physical address into the page ($8000 to $Bfff)

	tst	BANKING
	beq	banking_mode

	pulb				; get (A15-A12)
	pula				; get (A21-A16)

	lsld				; A14 is a 16K boundary and must be shifted twice
	lsld				; 

	staa	PPAGE			;this allow to select the page (A21-A14)

	pulb				; restore record length
	bra	LO_HALF

banking_mode
	pulb				; get (A15-A12)
	pula				; get (A21-A16)
	pulb				; restore record length

	ldx	S_ADR
	cpx	#$C000
	bhs	last_page

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	dec	PPAGE				;decrement PPAGE so that range is $8000 to $BFFF

	bra	LO_HALF

last_page
	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	bra	LO_HALF

Load1
	jsr	GetHexByte			;S1 record, so get byte count
	tab
	subb	#3				;adjust count for address and checksum bytes
	jsr	GetAddress			;get address

	cpy	#$c000				;if address is in last page
	bhs	yes_lastPage

	tst	BANKING
	bne	LO_HALF				;This mode only works for S1 type records in bank mode

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page
	dec	PPAGE				;decrement PPAGE so that range is $8000 to $BFFF

	bra	LO_HALF

yes_lastPage
	xgdy
	subd	#$4000				;Subtract $4000 to bring address to between $8000 to $BFFF
	xgdy

	movb	LAST_PPAGE,PPAGE		;$C000 to $FFFF is last page

LO_HALF
	dey				;adjust address for L1B loop

L1B					;repeat
	jsr	GetHexByte		;get a byte (in A)

	iny				;increment target address
	cpy	#FLASH			;if address below Flash then
	bhs	L1C

	cpy	#PPAGE			;if address same as PPAGE register then 
	bne	write10

	staa	0,y			;do normal write to PPAGE
	bra	VF

write10
	cpy	#$C00
	blo	check_ram

	staa	0,y			;do normal write to memory
	bra	VF

check_ram
	cpy	#$800
	bhs	write_temp

	staa	0,y			;do normal write to memory
	bra	VF

write_temp
	staa	temp			;do normal write to memory
	bra	VerfOK

L1C
	tst	WRITE_MODE		;else if not Verify then
	beq	VF

	tst	flashtype		;check if Flash or Ram
	beq	ram_wr
	
	bsr	WriteFlash		;do Flash write
	bra	flash_wr

ram_wr
	staa	0,y			;do normal write (it's RAM or register)
	bra	VF

flash_wr
	ldaa	BYTE

VF
	cmpa	0,y			;if byte doesn't verify then
	beq	VerfOK

	bset	PORTT,LED_ON		;turn on PT6 LED
	movb	#$ff,ERROR		;set error flag

	tst	first_err		;test if error already set
	bne	VerfOK

	movb	#$ff,first_err
	sty	abs_address
	movb	PPAGE,page_address	
	movb	0,y,bad_data		;save bad data
	staa	good_data		;save good data

VerfOK
	decb				;decrement # of bytes
	bne	L1B			;until all bytes in record programmed
	lbra	Get_S			;get next S-record

PSX
	rts

;==============================================================================
;---------------------
;Flash Burning routine
;---------------------
WriteFlash
        bset	PORTT,LED_ON	      	;PT6 LED on
        
	movb	PPAGE,CURPAGE		;preserve PPAGE value
	clr	PPAGE			;Flash needs to see address of $X555,

        brset   flashtype,%0001,amdflash        ;amd 512K
        brset   flashtype,%0010,amdflash        ;amd 128K
        brset   flashtype,%0100,stflash         ;st 512K
        brset   flashtype,%1000,stflash         ;st 128K
        bra     DPXXX

stflash
        movb    #$aa,$8555      	;need offset of $8000 to generate CSP0*
                                	;for Flash CE* then add $1555 ($5555-$4000)
        movb    #$55,$82aa      	;Flash needs address $2aaa (in first page)
					;so just add $8000 to generate CSP0* for CE*
        movb    #$a0,$8555
        bra     burn10        

amdflash
	inc	PPAGE			;so PPAGE=1 for $4000 offset into flash
	movb	#$aa,$9555		;need offset of $8000 to generate CSP0*
	clr	PPAGE			;for Flash CE* then add $1555 ($5555-$4000)
	movb	#$55,$aaaa		;Flash needs address $2aaa (in first page)
					;so just add $8000 to generate CSP0* for CE*
	inc	PPAGE			;same logic as before
	movb	#$a0,$9555

burn10
	movb	CURPAGE,PPAGE		;restore PPAGE value
	staa	0,y			;write Flash
	staa	BYTE		

DataPoll				;wait until it 'took'
	ldaa	BYTE			;get value being programmed
	eora	0,y
	bpl	DPP			;until DQ7 matches data bit7 or

CheckDQ5
	ldaa	#$20
	bita	0,y
	beq	DataPoll		;DQ5=1 (timeout failure)

DP_Fail:
	ldaa	BYTE			;check one last time before giving up
	eora	0,y
        bpl     DPXX

DPP:
	ldaa	BYTE			;verify data
	cmpa	0,y
        beq     DPXX

DPXXX
	bset	PORTT,LED_ON		;PT6 LED on
	movb	#$ff,ERROR
        bra     DPX

DPXX
        clr	PORTT   		;PT6 LED off

DPX:
	rts


;==============================================================================
;-------------------
;Erase Flash Routine
;-------------------
EraseFlash
	bset	PORTT,LED_ON	;LED on

	clr	ERROR
	clr	PPAGE

        brset   flashtype,%0001,erase_amd       ;erase AMD 512K memory
        brset   flashtype,%0010,erase_amd       ;erase AMD 128K memory
        brset   flashtype,%0100,erase_st        ;erase ST 512K memory
        brset   flashtype,%1000,erase_st        ;erase ST 128K memory
  
	bra     erase_f

erase_amd
	movb	#$1,PPAGE
	movb	#$aa,$9555
	clr	PPAGE
	movb	#$55,$aaaa
	movb	#$1,PPAGE
	movb	#$80,$9555
	movb	#$aa,$9555
	clr	PPAGE
	movb	#$55,$aaaa
	movb	#$1,PPAGE
	movb	#$10,$9555

        bra     erase10

erase_st
        movb    #$aa,$8555
        movb    #$55,$82aa
        movb    #$80,$8555
        movb    #$aa,$8555
        movb    #$55,$82aa
        movb    #$10,$8555

erase10
        clr     PPAGE
	ldy	#FLASH
	movb	#$ff,FLASH

EF1:
	ldab	0,y			;msb of Flash byte is low until erased
	bmi	EF_OK
	ldab	#$20			;DQ5=1 if write times out
	bitb	0,y
	beq	EF1

	ldab	0,y			;check once more
	bmi	EF_OK

erase_f
	ldx	#M_ERASE_FAIL		;  send FAIL message
	bra	EF_SS

EF_OK
	ldx	#M_ERASE_OK		;  send OK message

EF_SS
	clr	PORTT			;LED off
	jsr	OutStr

	rts
